package data.transaction;

import java.math.BigDecimal;

import qora.assets.Asset;
import qora.transaction.Transaction.TransactionType;

public class MessageTransactionData extends TransactionData {

	// Properties
	private int version;
	private byte[] senderPublicKey;
	private String recipient;
	private Long assetId;
	private BigDecimal amount;
	private byte[] data;
	private boolean isText;
	private boolean isEncrypted;

	// Constructors

	public MessageTransactionData(int version, byte[] senderPublicKey, String recipient, Long assetId, BigDecimal amount, byte[] data, boolean isText,
			boolean isEncrypted, BigDecimal fee, long timestamp, byte[] reference, byte[] signature) {
		super(TransactionType.MESSAGE, fee, senderPublicKey, timestamp, reference, signature);

		this.version = version;
		this.senderPublicKey = senderPublicKey;
		this.recipient = recipient;

		if (assetId != null)
			this.assetId = assetId;
		else
			this.assetId = Asset.QORA;

		this.amount = amount;
		this.data = data;
		this.isText = isText;
		this.isEncrypted = isEncrypted;
	}

	public MessageTransactionData(int version, byte[] senderPublicKey, String recipient, Long assetId, BigDecimal amount, byte[] data, boolean isText,
			boolean isEncrypted, BigDecimal fee, long timestamp, byte[] reference) {
		this(version, senderPublicKey, recipient, assetId, amount, data, isText, isEncrypted, fee, timestamp, reference, null);
	}

	// Getters/Setters

	public int getVersion() {
		return this.version;
	}

	public byte[] getSenderPublicKey() {
		return this.senderPublicKey;
	}

	public String getRecipient() {
		return this.recipient;
	}

	public Long getAssetId() {
		return this.assetId;
	}

	public BigDecimal getAmount() {
		return this.amount;
	}

	public byte[] getData() {
		return this.data;
	}

	public boolean getIsText() {
		return this.isText;
	}

	public boolean getIsEncrypted() {
		return this.isEncrypted;
	}

}
